ods graphics off;
ods html close;
ods noresults;
ods listing;

DATA TEST;
INPUT Subject COUNT;
CARDS;
1 1000
;RUN;

%MACRO simulation (NUMSETTINGS=);
%DO i = 1 %TO &NUMSETTINGS;
data test&i;
set test;
u1 = ranuni(1+1000*&i)*0.8+0.1; /* this parameter determines the prevalence of covariate1 */
u2 = (ranuni(2+1000*&i)*6)-3; /* this is the difference between the mean of covariate2 in patients with versus without covariate1 */
u3 = (ranuni(3+1000*&i)*4)-2; /* this is the coefficient for covariate1 in the propensity score model */
u4 = (ranuni(4+1000*&i)*4)-2; /* this is the coefficient for covariate2 in the propensity score model */
u5 = (ranuni(5+1000*&i)*2)-1; /* this is the coefficient for covariate1 in the Cox model */
u6 = (ranuni(6+1000*&i)*2)-1; /* this is the coefficient for covariate2 in the Cox model */
u7 = (ranuni(7+1000*&i)*2)-1; /* this is the coefficient for the treatment variable in the Cox model */
u8 = (ranuni(8+1000*&i)*1.5)+0.5; /* this is the Weibull shape parameter */
u9 = ranuni(9+1000*&i)*0.05+0.05;   /* this is the Weibull scale parameter (baseline hazard) */
u10 = ranuni(10+1000*&i)*2-1; /* this is the log-hazard ratio for the competing rate */ 
PS_int = ranuni(&i);
if PS_int < 0.33 then PS_intercept = -2;
else if PS_int > 0.66 then PS_intercept = 2;
else PS_intercept = 0; /* PS_intercept is the intercept from the propensity score (treatment probability) model */
options nosource nonotes;
run;

DATA test_cohort&i (DROP=subject count);
	SET TEST&i;
	DO i=1 TO COUNT;
	OUTPUT;
	END;
options nosource nonotes;
RUN;

data test_cohort&i (DROP=i);
set test_cohort&i;
Subject = i;
options nosource nonotes;
run;

%END;
ODS LISTING;

%MEND simulation;

%simulation(NUMSETTINGS=40);




%MACRO trial(NUMSAMP=, SETTING=);
	
ODS LISTING CLOSE;

%DO i = 1 %TO &NUMSAMP;
data realization&i;
set test_cohort&SETTING;
covariate1 = ranbin(&i*10000,1,u1);
if covariate1 = 1 then covariate2 = rannor(&i*20000)+u2;
else covariate2 = rannor(&i*30000);

treat_prob = 
exp(PS_intercept+u3*covariate1+u4*covariate2)/(1+exp(PS_intercept+u3*covariate1+u4*covariate2)); 

treatment = ranbin(&i*40000,1,treat_prob);

X =(-1*(Log(ranuni(&i*50000)))/(u9*EXP(u7*treatment + u5*covariate1 + u6*covariate2)))**(1/u8);

C = uniform(&i+60000)*2 + 3;
if treatment = 1 then H = ranexp(&i+70000)/0.05;
else H = ranexp(&i+80000)/(0.05*EXP(u10));
if X < C AND X < H then Y = 1;
else Y = 0;
T = min(X,C,H);
if H < C AND H < X then Z = 1;
else Z = 0;
if Y = 0 AND Z = 0 then S = 1;
else S = 0;
w = EXP(u7*treatment + u5*covariate1 + u6*covariate2);
options nosource nonotes;
run;


ods output ParameterEstimates = Oput_Cox&i;
proc phreg data=realization&i;
model T*Y(0)=treatment covariate1 covariate2 / risklimits;
options nosource nonotes;
run;

ods output Summary=treatment_count&i;
proc means data=realization&i sum;
var Y Z S T;
where treatment=1;
options nosource nonotes;
run;

data treatment_count&i (drop = Y_sum Z_sum S_sum T_sum);
set treatment_count&i;
treatment_count = Y_sum;
treatment_competing = Z_sum;
treatment_non_event = S_sum;
treatment_time = T_sum;
options nosource nonotes;
run;

ods output Summary=control_count&i;
proc means data=realization&i sum;
var Y Z T S;
where treatment=0;
options nosource nonotes;
run;

data control_count&i (drop = Y_sum Z_sum S_sum T_sum);
set control_count&i;
control_count = Y_sum;
control_competing = Z_sum;
control_non_event = S_sum;
control_time = T_sum;
options nosource nonotes;
run;

ods output  FitStatistics = Oput_Reg&i;
proc reg data=realization&i;
model treatment = covariate1 covariate2;
weight w;
options nosource nonotes;
run;
quit;

data Oput_Reg&i;
set Oput_Reg&i;
if Label2 ^= 'R-Square' then delete;
options nosource nonotes;
run;

data Oput_Reg&i (keep = cValue2);
set Oput_Reg&i;
options nosource nonotes;
run;



ods output  FitStatistics = Oput_unweighted_Reg&i;
proc reg data=realization&i;
model treatment = covariate1 covariate2;
options nosource nonotes;
run;
quit;

data Oput_unweighted_Reg&i;
set Oput_unweighted_Reg&i;
if Label2 ^= 'R-Square' then delete;
options nosource nonotes;
run;

data Oput_unweighted_Reg&i (keep = cValue2);
set Oput_unweighted_Reg&i;
options nosource nonotes;
run;

data Oput_unweighted_Reg&i (drop = cValue2);
set Oput_unweighted_Reg&i;
Unweighted_R_squared = cValue2*1;
options nosource nonotes;
run;

ods output Summary=treatment_prev&i;
proc means data=realization&i mean;
var treat_prob;
options nosource nonotes;
run;

data Oput&i;
merge Oput_Cox&i treatment_count&i control_count&i Oput_Reg&i Oput_unweighted_Reg&i treatment_prev&i test&SETTING;
options nosource nonotes;
run;

data Oput&i;
set Oput&i;
if Parameter ^= 'treatment' then delete;
Weighted_R_squared = cValue2*1;
if HRLowerCL < exp(u7) < HRUpperCL then coverage = 1;
else coverage = 0;
if u7 > 0 AND HRLowerCL > 1 then s_power = 1;
else if u7 < 0 AND HRUpperCL < 1 then s_power = 1;
else s_power  = 0;
if u7 < 0 AND HRLowerCL > EXP(-1) then n_power = 1;
else if u7 > 0 AND HRUpperCL < EXP(1) then n_power = 1;
else n_power  = 0;
options nosource nonotes;
run;

PROC APPEND BASE=DONE&SETTING NEW=OPUT&i FORCE;
RUN;


PROC DATASETS LIBRARY=WORK;
DELETE realization&i oput_Cox&i oput_Reg&i Oput_unweighted_Reg&i oput&i
treatment_count&i control_Count&i true_HR&i treatment_prev&i;
RUN;QUIT;


ods output Summary=Final&SETTING;
proc means data=done&SETTING n mean median std nolabels;
var treatment_count treatment_competing treatment_non_event treatment_time 
control_count control_competing control_non_event control_time
Weighted_R_squared Unweighted_R_squared
Estimate HazardRatio HRLowerCL HRUpperCL coverage StdErr 
s_power n_power u1 u2 u3 u4 u5 u6 u7 u8 u9 u10 treat_prob_Mean;
options nosource nonotes;
run;

data final&SETTING;
set final&SETTING;
little_r = (treat_prob_Mean_Mean)/(1-treat_prob_Mean_Mean);
total_events = treatment_count_Mean + control_count_Mean;
crude_RR = (treatment_count_Mean / treatment_time_Mean)/(control_count_Mean/control_time_mean);
little_r_prime = treatment_non_event_Mean / control_non_event_Mean;
little_r_tilde = (little_r + little_r_prime)/2;

SE0 = (SQRT(((little_r + 1)**2)/(total_events*little_r)))/StdErr_Mean;
SE0_adj = (SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean))/StdErr_Mean;

SE1 = (SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR)))/StdErr_Mean;
SE1_adj = (SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean))/StdErr_Mean;

SE2 = (SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR)))/StdErr_Mean;
SE2_adj = (SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean))/StdErr_Mean;

SE3 = (SQRT(1/treatment_count_Mean + 1/control_count_Mean))/StdErr_Mean;
SE3_adj = SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean))/StdErr_Mean;
options nosource nonotes;
run;

%END;
ODS LISTING;

%MEND trial;

%trial(NUMSAMP=2000, SETTING=1);

%trial(NUMSAMP=2000, SETTING=2);

%trial(NUMSAMP=2000, SETTING=3);

%trial(NUMSAMP=2000, SETTING=4);

%trial(NUMSAMP=2000, SETTING=5);

%trial(NUMSAMP=2000, SETTING=6);

%trial(NUMSAMP=2000, SETTING=7);

%trial(NUMSAMP=2000, SETTING=8);

%trial(NUMSAMP=2000, SETTING=9);

%trial(NUMSAMP=2000, SETTING=10);

%trial(NUMSAMP=2000, SETTING=11);

%trial(NUMSAMP=2000, SETTING=12);

%trial(NUMSAMP=2000, SETTING=13);

%trial(NUMSAMP=2000, SETTING=14);

%trial(NUMSAMP=2000, SETTING=15);

%trial(NUMSAMP=2000, SETTING=16);

%trial(NUMSAMP=2000, SETTING=17);

%trial(NUMSAMP=2000, SETTING=18);

%trial(NUMSAMP=2000, SETTING=19);

%trial(NUMSAMP=2000, SETTING=20);

%trial(NUMSAMP=2000, SETTING=21);

%trial(NUMSAMP=2000, SETTING=22);

%trial(NUMSAMP=2000, SETTING=23);

%trial(NUMSAMP=2000, SETTING=24); 

%trial(NUMSAMP=2000, SETTING=25);

%trial(NUMSAMP=2000, SETTING=26);

%trial(NUMSAMP=2000, SETTING=27);

%trial(NUMSAMP=2000, SETTING=28);

%trial(NUMSAMP=2000, SETTING=29);

%trial(NUMSAMP=2000, SETTING=30);

%trial(NUMSAMP=2000, SETTING=31);

%trial(NUMSAMP=2000, SETTING=32);

%trial(NUMSAMP=2000, SETTING=33);

%trial(NUMSAMP=2000, SETTING=34); 

%trial(NUMSAMP=2000, SETTING=35);

%trial(NUMSAMP=2000, SETTING=36);

%trial(NUMSAMP=2000, SETTING=37);

%trial(NUMSAMP=2000, SETTING=38);

%trial(NUMSAMP=2000, SETTING=39);

%trial(NUMSAMP=2000, SETTING=40);



data final;
set final1 final2 final3 final4 final5 final6 final7 final8 final9 final10
final11 final12 final13 final14 final15 final16 final17 final18 final19 final20
final21 final22 final23 final24 final25 final26 final27 final28 final29 final30
final31 final32 final33 final34 final35 final36 final37 final38 final39 final40;
run;

PROC DATASETS LIBRARY=WORK;
DELETE final1 final2 final3 final4 final5 final6 final7 final8 final9 final10
final11 final12 final13 final14 final15 final16 final17 final18 final19 final20
final21 final22 final23 final24 final25 final26 final27 final28 final29 final30
final31 final32 final33 final34 final35 final36 final37 final38 final39 final40

done1 done2 done3 done4 done5 done6 done7 done8 done9 done10
done11 done12 done13 done14 done15 done16 done17 done18 done19 done20
done21 done22 done23 done24 done25 done26 done27 done28 done29 done30
done31 done32 done33 done34 done35 done36 done37 done38 done39 done40

test1 test2 test3 test4 test5 test6 test7 test8 test9 test10
test11 test12 test13 test14 test15 test16 test17 test18 test19 test20
test21 test22 test23 test24 test25 test26 test27 test28 test29 test30
test31 test32 test33 test34 test35 test36 test37 test38 test39 test40

test_cohort1 test_cohort2 test_cohort3 test_cohort4 test_cohort5 test_cohort6 test_cohort7 test_cohort8 test_cohort9 test_cohort10
test_cohort11 test_cohort12 test_cohort13 test_cohort14 test_cohort15 test_cohort16 test_cohort17 test_cohort18 test_cohort19 test_cohort20
test_cohort21 test_cohort22 test_cohort23 test_cohort24 test_cohort25 test_cohort26 test_cohort27 test_cohort28 test_cohort29 test_cohort30
test_cohort31 test_cohort32 test_cohort33 test_cohort34 test_cohort35 test_cohort36 test_cohort37 test_cohort38 test_cohort39 test_cohort40
;
RUN;QUIT;

data final;
set final;
if treatment_count_Mean < 15 OR control_count_Mean < 15
then delete;
if Unweighted_R_squared_Mean > 0.5 then delete;
options nosource nonotes;
run;

data final;
set final;
true_HR = exp(u7_Mean);
SE0_adj_LCL = EXP(U7_Mean- 1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));
SE1_adj_LCL = EXP(U7_Mean- 1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
SE2_adj_LCL = EXP(U7_Mean- 1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
SE3_adj_LCL = EXP(U7_Mean- 1.96*SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));


SE0_adj_UCL = EXP(U7_Mean+ 1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));
SE1_adj_UCL = EXP(U7_Mean+ 1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
SE2_adj_UCL = EXP(U7_Mean+ 1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
SE3_adj_UCL = EXP(U7_Mean+ 1.96*SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));

if U7_Mean> 0 then
SE0_adj_s_power=1-cdf('NORMAL',0,U7_Mean-1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));
else SE0_adj_s_power=cdf('NORMAL',0,U7_Mean+1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean> 0 then
SE1_adj_s_power=1-cdf('NORMAL',0,U7_Mean-1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
else SE1_adj_s_power=cdf('NORMAL',0,U7_Mean+1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean> 0 then
SE2_adj_s_power=1-cdf('NORMAL',0,U7_Mean-1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
else SE2_adj_s_power=cdf('NORMAL',0,U7_Mean+1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean> 0 then
SE3_adj_s_power=1-cdf('NORMAL',0,U7_Mean-1.96*(SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean))),SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));
else SE3_adj_s_power=cdf('NORMAL',0,U7_Mean+1.96*(SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean))),SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));

s_power_sim = s_power_Mean;


if U7_Mean< 0 then
SE0_adj_n_power=1-cdf('NORMAL',-1,U7_Mean-1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));
else SE0_adj_n_power=cdf('NORMAL',1,U7_Mean+1.96*(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r + 1)**2)/(total_events*little_r))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean< 0 then
SE1_adj_n_power=1-cdf('NORMAL',-1,U7_Mean-1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
else SE1_adj_n_power=cdf('NORMAL',1,U7_Mean+1.96*(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r*crude_RR + 1)**2)/(total_events*little_r*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean< 0 then
SE2_adj_n_power=1-cdf('NORMAL',-1,U7_Mean-1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));
else SE2_adj_n_power=cdf('NORMAL',1,U7_Mean+1.96*(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)),(SQRT(((little_r_tilde*crude_RR + 1)**2)/(total_events*little_r_tilde*crude_RR))/SQRT(1-Weighted_R_squared_Mean)));

if U7_Mean< 0 then
SE3_adj_n_power=1-cdf('NORMAL',-1,U7_Mean-1.96*(SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean))),SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));
else SE3_adj_n_power=cdf('NORMAL',1,U7_Mean+1.96*(SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean))),SQRT((1/treatment_count_Mean + 1/control_count_Mean)/(1-Weighted_R_squared_Mean)));

n_power_sim = n_power_Mean;

comp_HR = 0.05 / (0.05*EXP(u10_Mean));

if crude_RR > 1 AND little_r > 1 then Direction = 1;
else if crude_RR < 1 AND little_r < 1 then Direction = 1;
else Direction = 0;

options nosource nonotes;
run;

data tables123_full_output;
set final;
if Direction = 0 then delete;
run;

data tables123_full_output;
set tables123_full_output;
Count = _N_;
run;

data tables123_full_output;
set tables123_full_output;
if Count > 15 then delete;
run;

proc sort data=tables123_full_output;
by Unweighted_R_squared_Mean;
run;


data tables456_full_output;
set final;
if Direction = 1 then delete;
run;

data tables456_full_output;
set tables456_full_output;
Count = _N_;
run;

data tables456_full_output;
set tables456_full_output;
if Count > 15 then delete;
run;

proc sort data=tables456_full_output;
by Unweighted_R_squared_Mean;
run;

data simulation_output;
set tables123_full_output tables456_full_output;
run;

data simulation_output;
set simulation_output;
s_power_sim_LCL = s_power_sim - 1.96*SQRT(s_power_sim*(1-s_power_sim)/Estimate_N);
s_power_sim_UCL = s_power_sim + 1.96*SQRT(s_power_sim*(1-s_power_sim)/Estimate_N);
n_power_sim_LCL = n_power_sim - 1.96*SQRT(n_power_sim*(1-n_power_sim)/Estimate_N);
n_power_sim_UCL = n_power_sim + 1.96*SQRT(n_power_sim*(1-n_power_sim)/Estimate_N);
SE_ratio = StdErr_Mean / Estimate_StdDev;
cov1_HR = EXP(u5_Mean);
cov2_HR = EXP(u6_Mean);
run;

ods html;

/* Table 3 */
title 'Table 3';
proc print data=simulation_output;
var Weighted_R_squared_Mean u8_Mean treatment_count_Mean treatment_competing_Mean
control_count_Mean control_competing_Mean little_r crude_RR true_HR cov1_HR cov2_HR 
comp_HR
SE0_adj SE1_adj SE2_adj SE3_adj SE_ratio;
run;

/* Supplementary table with confidence limits */
title 'Supplementary table with confidence limits';
proc print data=simulation_output;
var true_HR SE0_adj_LCL SE0_adj_UCL
SE1_adj_LCL SE1_adj_UCL
SE2_adj_LCL SE2_adj_UCL
SE3_adj_LCL SE3_adj_UCL
HRLowerCL_Mean HRUpperCL_Mean Coverage_Mean;
run;

/* Table 4 */
title 'Table 4';
proc print data=simulation_output;
var
SE0_adj_s_power SE1_adj_s_power SE2_adj_s_power SE3_adj_s_power s_power_sim s_power_sim_LCL s_power_sim_UCL
SE0_adj_n_power SE1_adj_n_power SE2_adj_n_power SE3_adj_n_power n_power_sim n_power_sim_LCL n_power_sim_UCL;
run;

libname lib 'H:\data\Documents\Methods papers 2017\Cox model power\Stat med second revision\Sim set 3';
run;

data lib.simulation_output;
set simulation_output;
run;
